///////////////////////////////////////////////////////////////////////////
//            Hyper Juke, Audio-CD database with media player.           //
//                Copyright (C) 1990,1996 Fumio Kawamata                 //
//                                                                       //
//  This program is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU General Public License as published by //
//  the Free Software Foundation; version 2 of the License.              //                                                    //
//                                                                       //
//  This program is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General    //
//  Public License in the directory "GNUGPL" for more details.           //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit Autoload;

interface

uses
  MyFunc, Disc,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, MPlayer, DB, DBTables, Grids, DBGrids,
  ExtCtrls;

type
  TformQuickRecall = class(TForm)
    panelBottom: TPanel;
    bitbtnCancel: TBitBtn;
    bitbtnOK: TBitBtn;
    panelTop: TPanel;
    queryDisc: TQuery;
    labelMessage: TLabel;
    datasourceDisc: TDataSource;
    dbgridDisc: TDBGrid;
    queryDiscDISC_ID: TStringField;
    queryDiscTITLE: TStringField;
    queryDiscTITLE2: TStringField;
    queryDiscPRODUCTION: TStringField;
    mediaplayerCD: TMediaPlayer;
    bitbtnSelect: TBitBtn;
    procedure bitbtnCancelClick(Sender: TObject);
    procedure bitbtnOKClick(Sender: TObject);
    procedure dbgridDiscDrawDataCell(Sender: TObject; const Rect: TRect;
      Field: TField; State: TGridDrawState);
    procedure bitbtnSelectClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  public
  end;

var
  formQuickRecall: TformQuickRecall;

implementation

{$R *.DFM}

uses Main;

//******************************************************************************
//* Shows details of disc
//******************************************************************************
{
procedure ShowDetailsOfDisc(DiscID:string; Sender: TObject);
begin
  if not(IsFormCreated('formDetailsOfDisc'))
    then
      begin
        DisplayStatus('Opening "Details of Disc"...');
        Application.CreateForm(TformDetailsOfDisc, formDetailsOfDisc);
        //formDetailsOfDisc.WindowState:=wsMinimized;
      end;

  with formDetailsOfDisc do begin
    queryDisc.Active:=False;
    queryDisc.Prepare;
    queryDisc.Close;
    queryDisc.Params[0].AsString:=DiscID;
    queryDisc.Open;
    queryDisc.Active:=True;
    queryMusic.Active:=False;
    queryMusic.Prepare;
    queryMusic.Close;
    queryMusic.Params[0].AsString:=DiscID;
    queryMusic.Open;
    queryMusic.Active:=True;
    Show;
    WindowState:=wsNormal;
  end;
end;
}


//******************************************************************************
//* Cancels quick recall
//******************************************************************************
procedure TformQuickRecall.bitbtnCancelClick(Sender: TObject);
begin
  Close;
end;


//******************************************************************************
//* Recalls current disc
//******************************************************************************
procedure TformQuickRecall.bitbtnOKClick(Sender: TObject);
const
  CRLF =chr($0D)+chr($0A);
var
  DiscID          : string;
  MessageStr      : string;
  Tracks          : Integer;
  Frames          : Longint;
begin
  mediaplayerCD.TimeFormat:=tfMSF;
  try
    mediaplayerCD.Open;
  except
    MessageStr:='Could not open the CD-Audio player.  '+ CRLF +
                'It may be occupied by another process.';
    MessageDlg(MessageStr, mtWarning, [mbOK],0);
    mediaplayerCD.Close;
    exit;
  end;
  Tracks:=mediaplayerCD.Tracks;
  Frames:=MyFunc.MSFtoFrame(mediaplayerCD.Length);
  with mediaplayerCD do begin
    if Error<>0 then begin
      MessageDlg('Error='+IntToStr(Error)+CRLF+
      'Error Message='+ErrorMessage, mtInformation, [mbOK], 0);
      mediaplayerCD.Close;
      exit;
    end;
  end;
  mediaplayerCD.Close;
  // Set SQL
  with queryDisc do begin
    Active:=False;
    Prepare;
    Close;
    ParamByName('TRACKS').AsInteger:=Tracks;
    ParamByName('FRAMES').AsInteger:=Frames;
    Open;
    Active:=True;
  end;
  // Displays dialog message when the disc data is not stored.
  case queryDisc.RecordCount of
    0 : begin
          MessageStr:='This CD is not registerd.'+CRLF+
           'Tracks: '+IntToStr(Tracks)+CRLF+
           'Frames: '+IntToStr(Frames);
          MessageDlg(MessageStr, mtInformation, [mbOk], 0);
        end;
    1 : begin
          formQuickRecall.Close;
          DiscID:=queryDisc.FieldByName('DISC_ID').AsString;
          //ShowDetailsOfDisc(DiscID, Sender);
          Main.CreateOrShowDetailsOfDiscForm(DiscID);
        end;
    else begin
      labelMessage.Caption:='Please select CD.';
      dbgridDisc.Visible:=True;
      bitbtnOK.Visible:=False;
      bitbtnOK.Enabled:=False;
      bitbtnSelect.Visible:=True;
      bitbtnSelect.Enabled:=True;
    end;
  end;
end;


//******************************************************************************
//* Draws data cell instead of default drawing method
//******************************************************************************
procedure TformQuickRecall.dbgridDiscDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
begin
  if gdSelected in State then begin
    dbgridDisc.Canvas.Brush.Color:=clAqua;
    dbgridDisc.Canvas.Font.Color:=clBlack;
    dbgridDisc.Canvas.Font.Style:=[fsBold];
  end;
  dbgridDisc.DefaultDrawDataCell(Rect, Field, State);
end;


//******************************************************************************
//* Select current disc
//******************************************************************************
procedure TformQuickRecall.bitbtnSelectClick(Sender: TObject);
var
  DiscID  : String;
begin
  formQuickRecall.Close;
  DiscID:=queryDisc.FieldByName('DISC_ID').AsString;
  //ShowDetailsOfDisc(DiscID, Sender);
  Main.CreateOrShowDetailsOfDiscForm(DiscID);
end;


procedure TformQuickRecall.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Main.DisplayStatus('Closed "Quick Recall" form.');
end;

end.
