///////////////////////////////////////////////////////////////////////////
//            Hyper Juke, Audio-CD database with media player.           //
//                Copyright (C) 1990,1997 Fumio Kawamata                 //
//                                                                       //
//  This program is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU General Public License as published by //
//  the Free Software Foundation; version 2 of the License.              //
//                                                                       //
//  This program is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General    //
//  Public License in the directory "GNUGPL" for more details.           //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit filter;

interface

uses Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, Messages, Dialogs, ExtCtrls, DBTables, DB, ComCtrls;

type
  TformFilter = class(TForm)
    bitbtnExecute           : TBitBtn;
    bitbtnReset             : TBitBtn;
    bitbtnRestrict          : TBitBtn;
    notebookItemList        : TNotebook;
    P0labelCandidates       : TLabel;
    P0labelSelected         : TLabel;
    P0listboxCandidates     : TListBox;
    P0listboxSelected       : TListBox;
    P0spdbtnExclude         : TSpeedButton;
    P0spdbtnExcludeAll      : TSpeedButton;
    P0spdbtnInclude         : TSpeedButton;
    P0spdbtnIncludeAll      : TSpeedButton;
    P1labelCandidates       : TLabel;
    P1labelSelected         : TLabel;
    P1listboxCandidates     : TListBox;
    P1listboxSelected       : TListBox;
    P1spdbtnExclude         : TSpeedButton;
    P1spdbtnExcludeAll      : TSpeedButton;
    P1spdbtnInclude         : TSpeedButton;
    P1spdbtnIncludeAll      : TSpeedButton;
    P2bitbtnSetTasteOver1   : TBitBtn;
    P2bitbtnSetTasteOver2   : TBitBtn;
    P2bitbtnSetTasteOver3   : TBitBtn;
    P2bitbtnSetTasteOver4   : TBitBtn;
    P2bitbtnSetTasteOver5   : TBitBtn;
    P2checkbox1: TCheckBox;
    P2checkbox2: TCheckBox;
    P2checkbox3: TCheckBox;
    P2checkbox4: TCheckBox;
    P2checkbox5: TCheckBox;
    P2labelSetTasteOver     : TLabel;
    P2labelTasteValue       : TLabel;
    P2PanelTaste            : TPanel;
    radiogroupListItem      : TRadioGroup;
    queryMusic: TQuery;
    progressbar1: TProgressBar;
    procedure bitbtnResetClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure P0spdbtnExcludeAllClick(Sender: TObject);
    procedure P0spdbtnExcludeClick(Sender: TObject);
    procedure P0spdbtnIncludeAllClick(Sender: TObject);
    procedure P0spdbtnIncludeClick(Sender: TObject);
    procedure P1spdbtnExcludeAllClick(Sender: TObject);
    procedure P1spdbtnExcludeClick(Sender: TObject);
    procedure P1spdbtnIncludeAllClick(Sender: TObject);
    procedure P1spdbtnIncludeClick(Sender: TObject);
    procedure radiogroupListItemClick(Sender: TObject);
    procedure bitbtnRestrictClick(Sender: TObject);
    procedure bitbtnExecuteClick(Sender: TObject);
  private
    function GetFirstSelection(List: TCustomListBox): Integer;
    procedure CopyListBoxItemsIntoCandidatesList;
    procedure ExcludeAllItems(var CandidatesList,SelectedList: TListBox);
    procedure ExcludeItem(var CandidatesList,SelectedList: TListBox);
    procedure IncludeAllItems(var CandidatesList,SelectedList: TListBox);
    procedure IncludeItem(var CandidatesList,SelectedList: TListBox);
    procedure MoveSelected(List: TCustomListBox; Items: TStrings);
    procedure ResetFilter(RestrictItemNo: integer);
    procedure SetButtons;
    procedure SetItem(List: TListBox; Index: Integer);
    procedure SetItemsIntoCandidateLists(RestrictNo: Integer);
  public
  end;

var
  formFilter: TformFilter;

implementation

uses Main, Musiclst;

{$R *.DFM}

const
  NoOfList = 6;

type
  StringArrayType = array[0..NoOfList-1] of TStringList;

var
  LastListNo : Integer; {Last Item Index of radiogroup1}
  CandidatesList : StringArrayType;
  SelectedList : StringArrayType;



{*************************************************************************
 * Gets filter item list.
 *************************************************************************}
procedure GetFilterItemList(ListNo: integer; var ItemList:TStringList);

procedure GetUnit_(NoOfItems: integer);
var
  DelimiterPos : integer;
  I            : integer;
  Str          : string;
  UnitIdStr    : string;
begin
  for I:=0 to NoOfItems-1 do
    begin
      Str:=SelectedList[ListNo].Strings[I];
      UnitIdStr:='';
      delete(Str,1,1); {Removes "(" character at the top of the string}
      DelimiterPos:=pos(') ',Str);
      if DelimiterPos>2
        then UnitIdStr:=copy(Str,1,DelimiterPos-1);
      ItemList.Add('"'+ UnitIdStr +'"');
    end;
end;


procedure GetDisc_(NoOfItems: integer);
var
  I            : integer;
  Str          : string;
begin
  for I:=0 to NoOfItems-1 do
    begin
      Str:=SelectedList[ListNo].Strings[I];
      while pos(' (',Str)>0 do {Removes " (" characters}
        begin
          delete(Str,1, pos(' (',Str)+1);
        end;
      delete(Str,length(Str),1); {Removes ")" character at the end of string.}
      ItemList.Add('"'+ Str +'"');
    end;
end;

procedure GetKeyword_(NoOfItems: integer);
var
  I            : integer;
  Str          : string;
begin
  for I:=0 to NoOfItems-1 do
    begin
      Str:=SelectedList[ListNo].Strings[I];
      while pos(' (',Str)>0 do {Removes " (" characters}
        delete(Str,1, pos(' (',Str)-1);
      delete(Str,length(Str)-1,1); {Removes ")" character at the end of string.}
      ItemList.Add('"%'+ Str +'%"');
    end;
end;

procedure GetTaste_;
var
  Str : string;
begin
  Str:='';
  if formFilter.P2checkbox1.Checked=True then Str:='1';
  if formFilter.P2checkbox2.Checked=True
    then
      begin
        if Str<>'' then Str:=Str+',';
        Str:=Str+'2';
      end;
  if formFilter.P2checkbox3.Checked=True
    then
      begin
        if Str<>'' then Str:=Str+',';
        Str:=Str+'3';
      end;
  if formFilter.P2checkbox4.Checked=True
    then
      begin
        if Str<>'' then Str:=Str+',';
        Str:=Str+'4';
      end;
  if formFilter.P2checkbox5.Checked=True
    then
      begin
        if Str<>'' then Str:=Str+',';
        Str:=Str+'5';
      end;
  ItemList.Add(Str);
end;

var
  I : integer;
  NoOfItems : integer; {Number of items in the item list variable.}

begin
  NoOfItems:=SelectedList[ListNo].Count;
  if NoOfItems>0
    then
      begin
        case ListNo of
          0: GetUnit_(NoOfItems);
          1: GetDisc_(NoOfItems);
          2,3:
             for I:=0 to NoOfItems-1 do
               ItemList.Add('"'+SelectedList[ListNo].Strings[I]+'"');
          4: GetKeyword_(NoOfItems);
          5: GetTaste_;
        end;
      end;
end;


procedure AddWhereCondition(ListNo:integer; var WhereList:TStringList);
//*************************************************************************
//* Sets items into candidate lists
//*************************************************************************
var
  ItemNo         : integer;
  Str            : string;
  NoOfItems : integer;
  WorkList       : TStringList;
begin
  WorkList:=TStringList.Create;
  GetFilterItemList(ListNo, WorkList);
  NoOfItems:=WorkList.Count;
  if NoOfItems=0 then Exit;

 // Str:='';
  //for ItemNo:=0 to NoOfItems-1 do Str:=Str+WorkList.Strings[ItemNo]+' , ';
 // MessageDlg('WorkList='+Str, mtInformation, [mbOK],0);



  if WhereList.Count=0 then WhereList.Add(' WHERE ');
  if WhereList.Count>1 then WhereList.Add(' AND ');

  case ListNo of
    0: Str:='"LOCATION.DB".UNIT_ID';
    1: Str:='"LOCATION.DB".DISC_ID';
    2: Str:='"MUSIC.DB".PLAYER';
    3: Str:='"MUSIC.DB".COMPOSER';
    4: Str:='"TASTE.DB".KEYWORD';
    5: Str:='"TASTE.DB".TASTE';
  end;

 case ListNo of
    0,1,2,3,5:
      begin
        WhereList.Add('('+Str+' IN (');
        for ItemNo:=0 to NoOfItems-1 do
          begin
            Str:=WorkList.Strings[ItemNo];
            WhereList.Add(Str);
            if ItemNo<(NoOfItems-1)
              then WhereList.Add(',');
          end;
        WhereList.Add('))');
      end;
    4:
      begin
        WhereList.Add('(');
        for ItemNo:=0 to (NoOfItems-1) do
          begin
            if ItemNo>0 then WhereList.Add(' OR ');
            WhereList.Add(Str+' LIKE '+WorkList.Strings[ItemNo]);
          end;
        WhereList.Add(')');
      end;
  end;
  WorkList.Free;
end;


procedure AddWhereSection(var Query:TQuery; List2: TStringList);
//*************************************************************************
//* Add WHERE section into SQL string list
//*************************************************************************
var
  I : integer;
  NoOfItems : integer;
  //Str : string;
begin
  NoOfItems:=List2.Count;
  if NoOfItems<1 then Exit;
  //Str:='';
  for I:=0 to (NoOfItems-1) do
    begin
      Query.SQL.Add(List2.Strings[I]);
      //Str:=Str+List2.Strings[I];
    end;
  //MessageDlg('WHERE SQL='+Str, mtInformation, [mbOK],0);
end;


procedure TformFilter.SetItemsIntoCandidateLists(RestrictNo: Integer);
//*************************************************************************
//* Sets items into candidate lists
//*************************************************************************
procedure AddSELECTsection_(ListNo: integer);
var
  Str : string;
begin
  queryMusic.SQL.Add('SELECT DISTINCT ');
  case ListNo of
    0 : Str:='"UNIT.DB".UNIT_ID, "UNIT.DB".TITLE';
    1 : Str:='"DISC.DB".DISC_ID, "DISC.DB".TITLE, "DISC.DB".TITLE2';
    2 : Str:='"MUSIC.DB".PLAYER';
    3 : Str:='"MUSIC.DB".COMPOSER';
    4 : Str:='"TASTE.DB".KEYWORD';
    5 : Str:='"TASTE.DB".TASTE';
  end;
  queryMusic.SQL.Add(Str+' ');
end;

procedure AddFROMsection_;
begin
  with queryMusic.SQL do
    begin
      Add('FROM ((("LOCATION.DB" INNER JOIN "MUSIC.DB"');
      Add('ON "LOCATION.DB".DISC_ID="MUSIC.DB".DISC_ID)');
      Add('INNER JOIN ":HyperJuke_P:TASTE.DB"');
      Add('ON ("MUSIC.DB".DISC_ID = ":HyperJuke_P:TASTE.DB".DISC_ID)');
      Add('AND("MUSIC.DB".TRACK1 = ":HyperJuke_P:TASTE.DB".TRACK1)');
      Add('AND("MUSIC.DB".TRACK2 = ":HyperJuke_P:TASTE.DB".TRACK2))');
      Add('INNER JOIN "UNIT.DB" ON "LOCATION.DB".UNIT_ID="UNIT.DB".UNIT_ID)');
      Add('INNER JOIN "DISC.DB" ON "LOCATION.DB".DISC_ID="DISC.DB".DISC_ID');
      Add(' ');
    end;
end;

procedure SetItemsIntoCandidatesList_(ListNo: integer);
var
  Item : string;
  N : integer;
  J : integer;
begin
  N:=queryMusic.RecordCount;
  queryMusic.First;

  for J:=1 to N do
    begin
      case ListNo of
        0: Item:='('+queryMusic.FieldByName('UNIT_ID').AsString +')  '+
                 queryMusic.FieldByName('TITLE').AsString;
        1: Item:=queryMusic.FieldByName('TITLE').AsString+' / '+
                 queryMusic.FieldByname('TITLE2').AsString+ '  ('+
                 queryMusic.FieldByName('DISC_ID').AsString+')';
        2: Item:=queryMusic.FieldByName('PLAYER').AsString;
        3: Item:=queryMusic.FieldByName('COMPOSER').AsString;
        4: Item:=queryMusic.FieldByName('KEYWORD').AsString;
      end;
      CandidatesList[ListNo].Add(Item);
      queryMusic.Next;
    end;
end;

var
  ListNo    : integer;
  N         : integer;
  WhereList      : TStringList;
begin
  WhereList:=TStringList.Create;
  Screen.Cursor:=crHourGlass;
  progressbar1.Max:=NoOfList-1;
  progressbar1.Visible:=True;
  for ListNo:=0 to (NoOfList-1) do
    begin
      progressbar1.Position:=ListNo;
      progressbar1.Update;
      CandidatesList[ListNo].Clear;
      if ListNo<>RestrictNo
        then
          begin
            SelectedList[ListNo].Clear;
            queryMusic.Active:=False;
            queryMusic.SQL.Clear;
            AddSELECTsection_(ListNo);
            AddFROMsection_;
            if RestrictNo<>-1
              then
                begin
                  WhereList.Clear;
                  AddWhereCondition(RestrictNo, WhereList);
                  AddWhereSection(queryMusic, WhereList);
                end;
            queryMusic.SQL.Add(';');
            queryMusic.Active:=True;
            N:=queryMusic.RecordCount;
            if N>0 then SetItemsIntoCandidatesList_(ListNo);
          end;
    end;
  progressbar1.Visible:=False;
  Screen.Cursor:=crDefault;
  WhereList.Free;
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.ResetFilter(RestrictItemNo: integer);
begin
  ///////////radiogroupListItem.ItemIndex:=0; {Note: This statement occurs onClick event.}
  //////////formFilter.Update;
  SetItemsIntoCandidateLists(RestrictItemNo);
  ////P0listboxCandidates.Items.Clear;
  //////P0listboxSelected.Items.Clear;
  ////P1listboxCandidates.Items.Clear;
  ///////P1listboxSelected.Items.Clear;
  if RestrictItemNo <> 5
    then
      begin
        P2checkbox1.Checked:=True;
        P2checkbox2.Checked:=True;
        P2checkbox3.Checked:=True;
        P2checkbox4.Checked:=True;
        P2checkbox5.Checked:=True;
      end;
  Case LastListNo of
     0,1:
        begin
          P0listboxCandidates.Items.Assign(CandidatesList[LastListNo]);
          P0listboxSelected.Items.Assign(SelectedList[LastListNo]);
          SetButtons;
       end;
     2,3,4:
        begin
          P1listboxCandidates.Items.Assign(CandidatesList[LastListNo]);
          P1listboxSelected.Items.Assign(SelectedList[LastListNo]);
          SetButtons;
      end;
  end;
  ////LastListNo:=0;
  ////notebookItemList.PageIndex:=0;
  SetButtons;
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.CopyListBoxItemsIntoCandidatesList;
begin
  case LastListNo of
    0,1:
      begin
        CandidatesList[LastListNo].Assign(P0listboxCandidates.Items);
        SelectedList[LastListNo].Assign(P0listboxSelected.Items);
      end;
    2,3,4:
      begin
        CandidatesList[LastListNo].Assign(P1listboxCandidates.Items);
        SelectedList[LastListNo].Assign(P1listboxSelected.Items);
     end;
  end;

end;



//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.radiogroupListItemClick(Sender: TObject);
var
  NewListNo : integer;
  NewNotePage : integer;
begin
  CopyListBoxItemsIntoCandidatesList;
  NewListNo:=radiogroupListItem.ItemIndex;
  case NewListNo of
    0,1:
      begin
        P0listboxCandidates.Items.Assign(CandidatesList[NewListNo]);
        P0listboxSelected.Items.Assign(SelectedList[NewListNo]);
        NewNotePage:=0;
      end;
    2,3,4:
      begin
        P1listboxCandidates.Items.Assign(CandidatesList[NewListNo]);
        P1listboxSelected.Items.Assign(SelectedList[NewListNo]);
        NewNotePage:=1;
      end;
    5:
      begin
        NewNotePage:=2;
      end;
  end;
  notebookItemList.PageIndex:=NewNotePage;
  LastListNo:=NewListNo;
  SetButtons;
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.bitbtnResetClick(Sender: TObject);
begin
  ResetFilter(-1);
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.bitbtnRestrictClick(Sender: TObject);
begin
  CopyListBoxItemsIntoCandidatesList;
  if SelectedList[LastListNo].Count=0 then Exit;
  ResetFilter(LastListNo);
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.IncludeItem(var CandidatesList,SelectedList: TListBox);
var
  Index: integer;
begin
  Index := GetFirstSelection(CandidatesList);
  MoveSelected(CandidatesList, SelectedList.Items);
  SetItem(CandidatesList, Index);
end;



//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.IncludeAllItems(var CandidatesList, SelectedList:TListBox);
var
  I: Integer;
begin
  for I := 0 to CandidatesList.Items.Count - 1 do
    SelectedList.Items.AddObject(CandidatesList.Items[I],
      CandidatesList.Items.Objects[I]);
  CandidatesList.Items.Clear;
  SetItem(CandidatesList, 0);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.ExcludeItem(var CandidatesList, SelectedList:TListBox);
var
  Index: Integer;
begin
  Index := GetFirstSelection(SelectedList);
  MoveSelected(SelectedList, CandidatesList.Items);
  SetItem(SelectedList, Index);
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.ExcludeAllItems(var CandidatesList, SelectedList:TListBox);
var
  I: Integer;
begin
  for I := 0 to SelectedList.Items.Count - 1 do
    CandidatesList.Items.AddObject(SelectedList.Items[I],
     SelectedList.Items.Objects[I]);
  SelectedList.Items.Clear;
  SetItem(SelectedList, 0);
end;





//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.MoveSelected(List: TCustomListBox; Items: TStrings);
var
  I: Integer;
begin
  for I := List.Items.Count - 1 downto 0 do
    if List.Selected[I] then
    begin
      Items.AddObject(List.Items[I], List.Items.Objects[I]);
      List.Items.Delete(I);
    end;
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.SetButtons;
var
  SrcEmpty, DstEmpty: Boolean;
begin
  case notebookItemList.PageIndex of
    0:
      begin
        SrcEmpty := P0listboxCandidates.Items.Count = 0;
        DstEmpty := P0listboxSelected.Items.Count = 0;
        P0spdbtnInclude.Enabled := not SrcEmpty;
        P0spdbtnIncludeAll.Enabled := not SrcEmpty;
        P0spdbtnExclude.Enabled := not DstEmpty;
        P0spdbtnExcludeAll.Enabled := not DstEmpty;
      end;
    1:
      begin
        SrcEmpty := P1listboxCandidates.Items.Count = 0;
        DstEmpty := P1listboxSelected.Items.Count = 0;
        P1spdbtnInclude.Enabled := not SrcEmpty;
        P1spdbtnIncludeAll.Enabled := not SrcEmpty;
        P1spdbtnExclude.Enabled := not DstEmpty;
        P1spdbtnExcludeAll.Enabled := not DstEmpty;
     end;
  end;
end;

//*************************************************************************
//*
//*************************************************************************
function TformFilter.GetFirstSelection(List: TCustomListBox): Integer;
begin
  for Result := 0 to List.Items.Count - 1 do
    if List.Selected[Result] then Exit;
  Result := LB_ERR;
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.SetItem(List: TListBox; Index: Integer);
var
  MaxIndex: Integer;
begin
  with List do
  begin
    SetFocus;
    MaxIndex := List.Items.Count - 1;
    if Index = LB_ERR then Index := 0
    else if Index > MaxIndex then Index := MaxIndex;
    Selected[Index] := True;
  end;
  SetButtons;
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.FormCreate(Sender: TObject);
var
  I : integer;
begin
  for I:=0 to NoOfList-1 do
    begin
      CandidatesList[I]:=TStringList.Create;
      SelectedList[I]:=TStringList.Create;
    end;
  notebookItemList.PageIndex:=0;
  LastListNo:=0;
  ResetFilter(-1);
end;


//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.FormDestroy(Sender: TObject);
var
  I : integer;
begin
  for I:=0 to NoOfList-1 do
    begin
      CandidatesList[I].Free;
      SelectedList[I].Free;
    end;
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P1spdbtnIncludeClick(Sender: TObject);
begin
  IncludeItem(P1listboxCandidates, P1listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P1spdbtnIncludeAllClick(Sender: TObject);
begin
  IncludeAllItems(P1listboxCandidates, P1listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P1spdbtnExcludeClick(Sender: TObject);
begin
  ExcludeItem(P1listboxCandidates, P1listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P1spdbtnExcludeAllClick(Sender: TObject);
begin
  ExcludeAllItems(P1listboxCandidates, P1listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P0spdbtnIncludeClick(Sender: TObject);
begin
  IncludeItem(P0listboxCandidates, P0listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P0spdbtnIncludeAllClick(Sender: TObject);
begin
  IncludeAllItems(P0listboxCandidates, P0listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P0spdbtnExcludeClick(Sender: TObject);
begin
  ExcludeItem(P0listboxCandidates, P0listboxSelected);
end;

//*************************************************************************
//*
//*************************************************************************
procedure TformFilter.P0spdbtnExcludeAllClick(Sender: TObject);
begin
  ExcludeAllItems(P0listboxCandidates, P0listboxSelected);
end;


procedure TformFilter.bitbtnExecuteClick(Sender: TObject);
var
 I : integer;
  SqlStr : string;
  ListNo    : integer;
  WhereList      : TStringList;
begin
  CopyListBoxItemsIntoCandidatesList;
  WhereList:=TStringList.Create;
  Screen.Cursor:=crHourGlass;
  //progressbar1.Max:=NoOfList-1;
  //progressbar1.Visible:=True;
  Main.CreateOrShowMusicListForm;
  with formMusicList.queryMusic do
    begin
      Active:=False;
      SQL.Clear;
      SQL.Add('SELECT "LOCATION.DB".UNIT_ID, "LOCATION.DB".SLOT_ID, ');
      SQL.Add('"MUSIC.DB".TRACK1, "MUSIC.DB".TRACK2, ');
      SQL.Add('"MUSIC.DB".TITLE, "MUSIC.DB".PLAYER, "MUSIC.DB".COMPOSER, "MUSIC.DB".FRAMES, ');
      SQL.Add('"TASTE.DB".KEYWORD, "TASTE.DB".TASTE, "TASTE.DB".COUNTER, "LOCATION.DB".DISC_ID ');

      SQL.Add(' FROM ( "LOCATION.DB" INNER JOIN "MUSIC.DB"');
      SQL.Add('ON "LOCATION.DB".DISC_ID="MUSIC.DB".DISC_ID)');
      SQL.Add('INNER JOIN ":HyperJuke_P:TASTE.DB"');
      SQL.Add(' ON ("MUSIC.DB".DISC_ID = ":HyperJuke_P:TASTE.DB".DISC_ID)');
      SQL.Add(' AND ("MUSIC.DB".TRACK1 = ":HyperJuke_P:TASTE.DB".TRACK1)');
      SQL.Add(' AND ("MUSIC.DB".TRACK2 = ":HyperJuke_P:TASTE.DB".TRACK2)');
      SQL.Add(' ');
    end;
  for ListNo:=0 to (NoOfList-1) do
    begin
      //progressbar1.Position:=ListNo;
      //progressbar1.Update;
      AddWhereCondition(ListNo, WhereList);
    end;
  AddWhereSection(formMusicList.queryMusic, WhereList);
  formMusicList.queryMusic.SQL.Add(' ORDER BY UNIT_ID, SLOT_ID, TRACK1, TRACK2;');

SqlStr:='';
for I:=0 to formMusicList.queryMusic.SQL.Count-1 do
  SqlStr:=SqlStr+formMusicList.queryMusic.SQL.Strings[I];
//MessageDlg('SQL='+SqlStr, mtInformation, [mbOK],0);

  formMusicList.queryMusic.Active:=True;
  //progressbar1.Visible:=False;
  Screen.Cursor:=crDefault;
  WhereList.Free;
end;

end.
