///////////////////////////////////////////////////////////////////////////
//                  Functions for the Borland Delphi                     //
//                Copyright (C) 1990,1996 Fumio Kawamata                 //
//                                                                       //
//  This library is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU Library General Public License as      //
//  published by the Free Software Foundation; version 2 of the License. //
//                                                                       //
//  This library is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library    //
//  General Public License in the directory "GNUGPL" for more details.   //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit MyFunc;

interface

uses
  SysUtils,WinTypes,Forms,Dialogs;

function AddMSFTime(A,B: string):string;
function AskYesNo(Msg:string):word;
function AskYesCancel(Msg:string):word;
function AskYesNoCancel(Msg:string):word;
function ExtractFileNameWithoutExt(FullPath:string):string;
function FrameToTime(InFrames:Longint):string;
function IsFormCreated(FormName:string):boolean;
function GetFormNumberByCaption(CaptionString: string): integer;
function MSFtoFrame(MSF:Longint):Longint;
function MSFtoTime(MSF:Longint):string;
function RemoveSpace(InStr:string):string;
function ShiftWord(var S:string; Separator:string):string;
function Spacing(N:word):string;
function TabToSpace(S:string):string;
function TMSFtoTime(TMSF:Longint):string;
function TMSFtoTrack(TMSF:Longint):string;
function Trim(InStr:string):string;
function UpCaseString(InStr:string):string;

implementation

type
  MSFRec = record
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
    NotUsed : byte;
  end;
  TMSFRec = record
    Tracks  : byte;
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
  end;

//*************************************************************************
//* Add MSF formatted time values
//*   - MSF means Minutes, Seconds, and Frames(Frame=1/75 seconds.)
//*************************************************************************
function AddMSFTime(A,B: string):string;
var
  F1   : Longint;
  F2   : Longint;
  Minutes         : String;
  Seconds : String;
  Frames  : String;
  F    : Longint;
begin
  F1:=StrToInt(copy(A,1,2))*4500+StrToInt(copy(A,4,2))*75+StrToInt(copy(A,7,2));
  F2:=StrToInt(copy(B,1,2))*4500+StrToInt(copy(B,4,2))*75+StrToInt(copy(B,7,2));
  F:=F1+F2;
  Minutes:=IntToStr(F div 4500); F:=F mod 4500;
  Seconds:=IntToStr(F div 75); F:=F mod 75;
  Frames:=IntToStr(F);
  if Length(Minutes)=1 then Minutes:='0'+Minutes;
  if Length(Seconds)=1 then Seconds:='0'+Seconds;
  if Length(Frames)=1 then Frames:='0'+Frames;
  AddMSFTime:=Minutes+':'+Seconds+':'+Frames;
end;


//*************************************************************************
//* Asks Yes or No using the message dialog
//*   - Returns mbYes or mbNo
//*************************************************************************
function AskYesNo(Msg:string):word;
begin
  AskYesNo:=MessageDlg(Msg, mtConfirmation, [mbYes,mbNo],0);
end;


//*************************************************************************
//* Asks Yes or Cancel using the message dialog
//*   - Returns mbYes or mbCancel
//*************************************************************************
function AskYesCancel(Msg:string):word;
begin
  AskYesCancel:=MessageDlg(Msg, mtConfirmation, [mbYes,mbCancel],0);
end;


//*************************************************************************
//* Asks Yes, No, or Cancel using the message dialog
//*   - Returns mbYes, mbNo, or mbCancel
//*************************************************************************
function AskYesNoCancel(Msg:string):word;
begin
  AskYesNoCancel:=MessageDlg(Msg, mtConfirmation, [mbYes,mbNo,mbCancel],0);
end;


//*************************************************************************
//* Extracts full path into the file name without extention
//*   - Example:
//*       ExtractFileNameWithoutExt('C:\somedir\somefile.ext')
//*       returns 'somefile'.
//*************************************************************************
function ExtractFileNameWithoutExt(FullPath:string):string;
var
    Ans             : string;
    Ext             : string;
    FileNameWithExt : string;
    L               : word;
begin
  FileNameWithExt:=ExtractFileName(FullPath);
  Ext:=ExtractFileExt(FullPath);
  if Ext='' then Ans:=FileNameWithExt
  else if FileNameWithExt='' then Ans:=''
  else begin
    L:=Length(FileNameWithExt)-Length(Ext);
    Ans:=copy(FileNameWithExt,1,L);
  end;
  ExtractFileNameWithoutExt:=Ans;
end;


//*************************************************************************
//* Converts frames to time
//*************************************************************************
function FrameToTime(InFrames:Longint):string;
var
  Frames : Longint;
  Min    : integer;
  MinStr : string;
  Hour   : integer;
  HourStr : string;
  Sec    : Integer;
  SecStr : string;
begin
  Frames:=InFrames;
  {* Hours}
  Hour:=Frames div 270000;
  if Hour>0
    then HourStr:=IntToStr(Hour)+'h'
    else HourStr:='';
  Frames:=Frames mod 270000;
  {* Minutes}
  Min:=Frames div 4500;
  MinStr:=IntToStr(Min);
  if (Hour>0) and (Min<10) then MinStr:='0'+MinStr;
  MinStr:=MinStr+'m';
  Frames:=Frames mod 4500;
  {* Seconds}
  Sec:=Frames div 75;
  Frames:=Frames mod 75;
  if Frames>37 then inc(Sec);
  SecStr:=IntToStr(Sec);
  if Length(SecStr)=1 then SecStr:='0'+SecStr;
  SecStr:=SecStr+'s';
  {* Setting the return value}
  FrameToTime:=HourStr+MinStr+SecStr;
end;

//*************************************************************************
//* Checks form is created or not
//*************************************************************************
function IsFormCreated(FormName:string):boolean;
var
  Matched : boolean;
  I : integer;
begin
  I:=0; Matched:=False;
  while (I<=(Screen.FormCount-1)) and (not(Matched)) do
    begin
      if Screen.Forms[I].Name=FormName then Matched:=True;
      inc(I);
    end;
  IsFormCreated:=Matched;
end;


//*************************************************************************
//* Gets form number
//*************************************************************************
function GetFormNumberByCaption(CaptionString: string): integer;
var
  Ans      : integer;
  Matched  : boolean;
  I        : integer;
begin
  I:=0; Matched:=False; Ans:=-1;
  while (I<=(Screen.FormCount-1)) and (not(Matched)) do
    begin
      if Screen.Forms[I].Caption=CaptionString
        then
          begin
            Matched:=True;
            Ans:=I;
          end;
      inc(I);
    end;
  GetFormNumberByCaption:=Ans;
end;


//*************************************************************************
//* Converts MSF value into Frames
//*************************************************************************
function MSFtoFrame(MSF:Longint):Longint;
var
  Ans : Longint;
  M   : Longint;
  S   : Integer;
  F   : Integer;
begin
  with MSFRec(MSF) do begin
    M:=Minutes;
    S:=Seconds;
    F:=Frames;
  end;
  Ans:=M*4500+S*75+F;
  MSFtoFrame:=Ans;
end;


//*************************************************************************
//* Converts MSF to Time
//*************************************************************************
function MSFtoTime(MSF:Longint):string;
var
  Str     : string;
  TimeStr : string;
begin
  with MSFRec(MSF) do begin
    // labelTrackValue.Caption:=Format('%2.d',[Track]);
    // Upper statement did not work correctly.
    Str:=IntToStr(Minutes);
    if Length(Str)=1 then Str:='0'+Str;
    TimeStr:=Str;
    Str:=IntToStr(Seconds);
    if Length(Str)=1 then Str:='0'+Str;
    TimeStr:=TimeStr+':'+Str;
    Str:=IntToStr(Frames);
    if Length(Str)=1 then Str:='0'+Str;
    TimeStr:=TimeStr+':'+Str;
  end;
  MSFtoTime:=TimeStr;
end;


//*************************************************************************
//* Removes space code in the string
//*************************************************************************
function RemoveSpace(InStr:string):string;
var
  Ans    : string;
  I      : word;
  L      : word;
  TestChar  : String[1];
begin
  Ans:='';
  L:=Length(InStr);
  if L>0 then
    begin
    for I:=1 to L do
      begin
      TestChar:=copy(InStr,I,1);
        if TestChar<>' ' then Ans:=Ans+TestChar;
        end;
  end;
  RemoveSpace:=Ans;
end;


//*************************************************************************
//* Shifts string left until separator character
//*   - String variable S holds 'This is a pen.'
//*     ShiftWord_(S,' ') returns 'This'.
//*     After execution, S holds 'is a pen.'.
//*************************************************************************
function ShiftWord(var S:string; Separator:string):string;
var
  L  : word;
  P  : word;
begin
  S:=Trim(S);
  P:=Pos(Separator,S);
  L:=Length(Separator);
  if P=0 then
    begin
    ShiftWord:=S;
    S:='';
    exit;
  end;
  if P=1 then
    begin
    ShiftWord:='';
    Delete(S,1,L);
    exit;
  end;
  ShiftWord:=copy(S,1,P-1);
  Delete(S,1,P-1); // Delete shifted part string
  Delete(S,1,L); // Delete separator string
end;


//*************************************************************************
//* Returns N spaces
//*************************************************************************
function Spacing(N:word):string;
var
  I : word;
  S : string;
begin
  S:='';
  if N>0 then for I:=1 to N do S:=S+' ';
  Spacing:=S;
end;


//*************************************************************************
//* Converts TAB code to SPACE code
//*************************************************************************
function TabToSpace(S:string):string;
var
  Ans  : string;
  C    : string;
  I    : word;
  L    : word;
begin
  Ans:='';
  L:=Length(S);
  if L>0 then
  begin
    for I:=1 to L do
      begin
      C:=copy(S,I,1);
      if C=chr($09) then C:=' ';
      Ans:=Ans+C;
    end;
  end;
  TabToSpace:=Ans;
end;


//*************************************************************************
//* Converts TMSF formatted value to Time
//*************************************************************************
function TMSFtoTime(TMSF:Longint):string;
var
  MSF : Longint;
begin
  MSF:=TMSF div 256;
  TMSFtoTime:=MSFtoTime(MSF);
end;


//*************************************************************************
//* Converts TMSF to Track
//*************************************************************************
function TMSFtoTrack(TMSF:Longint):string;
var
  TrackStr : string;
begin
  TrackStr:=IntToStr(TMSFRec(TMSF).Tracks);
  if Length(TrackStr)=1 then TrackStr:='0'+TrackStr;
  TMSFtoTrack:=TrackStr;
end;


//*************************************************************************
//* Removes space codes at the head and tail of string
//*   - Trim('  This is a pen   ') returns 'This is a pen'.
//*************************************************************************
function Trim(InStr:string):string;
var
  I          : word;
  L          : word;
  NonSpace  : boolean;
  S          : string;
begin
  S:=InStr;
  L:=Length(S);
  if (L=0)or(S=' ') then
    begin
      Trim:='';
    exit;
  end;

  // Removes spaces at the head part of the string
  I:=0;
  NonSpace:=False;
  while (NonSpace=False)and(I<L) do
    begin
    inc(I);
    if copy(S,I,1)<>' ' then NonSpace:=True;
  end;
  if NonSpace=True then
    begin
    if I>=2 then S:=copy(S,I,L-I+1);
  end;

  if Length(S)=1 then
  begin
    Trim:=S;
    exit;
    end;

  // Removes spaces at the tail part of the string
  I:=Length(S)+1;
  NonSpace:=False;
  while (NonSpace=False)and(I>1) do
    begin
    dec(I);
    if copy(S,I,1)<>' ' then NonSpace:=True;
  end;
  if NonSpace=True then S:=copy(S,1,I);

  Trim:=S;
end;


//*************************************************************************
//* Upcases string
//*   - UpCaseString('This is a pen.') returns 'THIS IS A PEN.'.
//*************************************************************************
function UpCaseString(InStr:string):string;
var
  Ans  : string;
  I    : word;
  L    : word;
begin
  Ans:=InStr;
  L:=Length(InStr);
  if L>0 then
    for I:=1 to L do Ans[I]:=UpCase(Ans[I]);
  UpCaseString:=Ans;
end;


end.

