///////////////////////////////////////////////////////////////////////////
//            Hyper Juke, Audio-CD database with media player.           //
//                Copyright (C) 1990,1996 Fumio Kawamata                 //
//                                                                       //
//  This program is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU General Public License as published by //
//  the Free Software Foundation; version 2 of the License.              //                                                    //
//                                                                       //
//  This program is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General    //
//  Public License in the directory "GNUGPL" for more details.           //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit progplayer;

interface

uses
  MPlayer, MyFunc,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, DB, DBTables, Grids, DBGrids,
  DBCtrls;

type
  TformProgPlayer = class(TForm)
    bitbtnClose             : TBitBtn;
    bitbtnDetailsOfMusic    : TBitBtn;
    bitbtnEject             : TBitBtn;
    bitbtnPause             : TBitBtn;
    bitbtnPlay              : TBitBtn;
    bitbtnStop              : TBitBtn;
    bitbtnTrackBwd          : TBitBtn;
    bitbtnTrackFwd          : TBitBtn;
    datasourceProg          : TDataSource;
    editStatus              : TEdit;
    labelTime               : TLabel;
    labelTimeValue          : TLabel;
    labelTrack              : TLabel;
    labelTrackValue         : TLabel;
    mediaplayerCD           : TMediaPlayer;
    panelBottom             : TPanel;
    panelPlayerButtons      : TPanel;
    panelStatus             : TPanel;
    panelTop                : TPanel;
    tableMusic              : TTable;
    tableProg               : TTable;
    tableProgDISC_ID        : TStringField;
    tableProgNO             : TSmallintField;
    tableProgSLOT_ID        : TStringField;
    tableProgTRACK1         : TSmallintField;
    tableProgTRACK2         : TSmallintField;
    tableProgUNIT_ID        : TStringField;
    tableProgWORK_PLAYER    : TStringField;
    tableProgWORK_TITLE     : TStringField;
    timer1                  : TTimer;
    radiogroupPage: TRadioGroup;
    tableProgPAGE: TStringField;
    dbgridProg: TDBGrid;
    panelMusicTime: TPanel;
    labelMusics: TLabel;
    labelNoOfMusicsValue: TLabel;
    labelTotalTimeValue: TLabel;
    tableProgWORK_FRAMES: TIntegerField;
    dbnavigator1: TDBNavigator;
    procedure bitbtnCloseClick(Sender: TObject);
    procedure bitbtnEjectClick(Sender: TObject);
    procedure bitbtnPauseClick(Sender: TObject);
    procedure bitbtnPlayClick(Sender: TObject);
    procedure bitbtnStopClick(Sender: TObject);
    procedure bitbtnTrackBwdClick(Sender: TObject);
    procedure bitbtnTrackFwdClick(Sender: TObject);
    procedure dbgridProgColExit(Sender: TObject);
    procedure dbgridProgDrawDataCell(Sender: TObject; const Rect: TRect; Field: TField; State: TGridDrawState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure mediaplayerCDNotify(Sender: TObject);
    procedure tableProgCalcFields(DataSet: TDataset);
    procedure timer1Timer(Sender: TObject);
    procedure bitbtnDetailsOfMusicClick(Sender: TObject);
    procedure radiogroupPageClick(Sender: TObject);
    procedure tableProgAfterPost(DataSet: TDataSet);
    procedure DisplayMusicTime;
  private
    procedure PlayCD;
    procedure PlayNext;
    procedure PlayNextAuto;
    procedure StopCD;
  public
  end;

var
  formProgPlayer: TformProgPlayer;

implementation

{$R *.DFM}

uses Main;

type
  MSFRec = record
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
    NotUsed : byte;
  end;
  TMSFRec = record
    Track   : byte;
    Minutes : byte;
    Seconds : byte;
    Frames  : byte;
  end;
var
  EndPosTMSF         : Longint;
  Paused             : Boolean;


procedure TformProgPlayer.DisplayMusicTime;
var
  I          : integer;
  NoOfMusics : integer;
  TotalFrames : longint;
  TotalTime  : string;
begin
  NoOfMusics:=tableProg.RecordCount;
  TotalFrames:=0;
  for I:=1 to NoOfMusics do
    begin
      TotalFrames:=TotalFrames
        + tableProg.FieldByName('WORK_FRAMES').AsInteger;
    end;
  TotalTime:=MyFunc.FrameToTime(TotalFrames);
  labelNoOfMusicsValue.Caption:=IntToStr(NoOfMusics);
  labelTotalTimeValue.Caption:=TotalTime;
  labelNoOfMusicsValue.Update;
  labelTotalTimeValue.Update;
//MessageDlg('Updated Music Time', mtInformation, [mbOK], 0);
end;


//*************************************************************************
//* Plays next track of the CdAudio player
//*************************************************************************
procedure TformProgPlayer.PlayNext;
begin
  tableProg.Next;
  if tableProg.Eof
    then
      begin
        StopCD;
      end
    else
      begin
        PlayCD;
      end;
end;


//*************************************************************************
//* Plays next track of the CdAudio automatically
//*************************************************************************
procedure TformProgPlayer.PlayNextAuto;
begin
  tableProg.Next;
  if not(tableProg.Eof) then begin
    PlayCD;
  end
  else begin
    StopCD;
  end;
end;


//*************************************************************************
//* Plays CdAudio player
//*************************************************************************
procedure TformProgPlayer.PlayCD;
var
  MSF            : Longint;
  StartPosTMSF   : Longint;
  T1             : byte;
  T2             : byte;

begin
  T1:=tableProg.FieldByName('Track1').AsInteger;
  T2:=tableProg.FieldByName('Track2').AsInteger;
  StartPosTMSF:=T1;
  mediaplayerCD.TimeFormat:=tfMSF;
  MSF:=mediaplayerCD.TrackLength[T2];
  TMSFRec(EndPosTMSF).Track:=T2;
  TMSFRec(EndPosTMSF).Minutes:=MSFRec(MSF).Minutes;
  TMSFRec(EndPosTMSF).Seconds:=MSFRec(MSF).Seconds;
  TMSFRec(EndPosTMSF).Frames:=MSFRec(MSF).Frames;
  with TMSFRec(EndPosTMSF) do begin
    editStatus.Text:='Check at Track '+IntToStr(Track)+', Time '+
    IntToStr(Minutes)+':'+IntToStr(Seconds);
  end;
  with mediaplayerCD do begin
    TimeFormat:=tfTMSF;
    StartPos:=StartPosTMSF;
    EndPos:=EndPosTMSF;
    Notify:=True;
    Play;
  end;
  Paused:=False;
  bitbtnPlay.Enabled:=False;
  bitbtnPause.Enabled:=True;
  bitbtnStop.Enabled:=True;
  bitbtnTrackFwd.Enabled:=True;
  bitbtnTrackBwd.Enabled:=True;
  bitbtnEject.Enabled:=False;
end;

//*************************************************************************
//* Stops CdAudio player
//*************************************************************************
procedure TformProgPlayer.StopCD;
begin
  mediaplayerCD.Rewind;
  mediaplayerCD.Stop;
  Paused:=False;
  bitbtnPlay.Enabled:=True;
  bitbtnPause.Enabled:=False;
  bitbtnStop.Enabled:=False;
  bitbtnTrackFwd.Enabled:=False;
  bitbtnTrackBwd.Enabled:=False;
  bitbtnEject.Enabled:=True;
end;

//*************************************************************************
//* Plays CdAudio player
//*************************************************************************
procedure TformProgPlayer.bitbtnPlayClick(Sender: TObject);
begin
  PlayCD;
end;


//*************************************************************************
//* Pauses CdAudio player
//*************************************************************************
procedure TformProgPlayer.bitbtnPauseClick(Sender: TObject);
begin
  mediaplayerCD.Pause;
  if Paused=True then begin
    mediaplayerCD.EndPos:=EndPosTMSF;
    mediaplayerCD.Play;
    Paused:=False;
    bitbtnPlay.Enabled:=False;
    bitbtnPause.Enabled:=True;
    bitbtnStop.Enabled:=True;
    bitbtnTrackFwd.Enabled:=True;
    bitbtnTrackBwd.Enabled:=True;
    bitbtnEject.Enabled:=False;
  end
  else begin
    Paused:=True;
    bitbtnPlay.Enabled:=False;
    bitbtnPause.Enabled:=True;
    bitbtnStop.Enabled:=False;
    bitbtnTrackFwd.Enabled:=False;
    bitbtnTrackBwd.Enabled:=False;
    bitbtnEject.Enabled:=False;
  end;
end;


//*************************************************************************
//* Stops CdAudio player
//*************************************************************************
procedure TformProgPlayer.bitbtnStopClick(Sender: TObject);
begin
  StopCD;
end;


//*************************************************************************
//* Plays next program
//*************************************************************************
procedure TformProgPlayer.bitbtnTrackFwdClick(Sender: TObject);
begin
   PlayNext;
end;


//*************************************************************************
//* Playes previous program
//*************************************************************************
procedure TformProgPlayer.bitbtnTrackBwdClick(Sender: TObject);
begin
  tableProg.Prior;
  if not(tableProg.Bof) then begin
    PlayCD;
  end;
end;


//*************************************************************************
//* Ejects CdAudio player
//*************************************************************************
procedure TformProgPlayer.bitbtnEjectClick(Sender: TObject);
begin
  mediaplayerCD.Eject;
  mediaplayerCD.Rewind;
  Paused:=False;
end;


//*************************************************************************
//* Executes interval timer
//*************************************************************************
procedure TformProgPlayer.timer1Timer(Sender: TObject);
const
  ModeStr   : array[TMPModes] of string[10]
              =('Not Ready','Stopped','Playing','Recording','Seeking',
                'Paused', 'Open');
var
  TimeStr   : String;
  TrackStr  : String;
  TMSF      : Longint;
begin
  if mediaplayerCD.Mode in [mpNotReady, mpOpen] then begin
    TrackStr:='--';
    TimeStr:='BUSY';
  end
  else begin
    TMSF:=mediaplayerCD.Position;
    TrackStr:=TMSFtoTrack(TMSF);
    TimeStr:=copy(TMSFtoTime(TMSF),1,5);
  end;
    labelTrackValue.Caption:=TrackStr;
    labelTimeValue.Caption:=TimeStr;
end;


//*************************************************************************
//* Executes when form close event occurs
//*************************************************************************
procedure TformProgPlayer.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  mediaplayerCD.Stop;
  timer1.Enabled:=False;
  Main.DisplayStatus('Closed "Player" form.');
  Release;
end;


//*************************************************************************
//* Close myself
//*************************************************************************
procedure TformProgPlayer.bitbtnCloseClick(Sender: TObject);
begin
  Close;
end;



//*************************************************************************
//* Executes when CDNotify event occurs
//*************************************************************************
procedure TformProgPlayer.mediaplayerCDNotify(Sender: TObject);
var
  ResultStr  : string;
begin
  case mediaplayerCD.NotifyValue of
    nvSuccessful : ResultStr:='Success';
    nvSuperseded : ResultStr:='Superseded';
    nvAborted    : ResultStr:='Aborted';
    nvFailure    : ResultStr:='Failure';
    else ResultStr:='UnKnown';
  end;
  //MessageDlg('Result Str='+ResultStr, mtInformation, [mbOK],0);
  if mediaplayerCD.NotifyValue=nvSuccessful then begin
    PlayNext;
  end;
end;


//*************************************************************************
//* Sets items into TITLE and PLAYER fields
//*************************************************************************
procedure TformProgPlayer.tableProgCalcFields(DataSet: TDataset);
var
  Key1 : String;
  Key2 : Integer;
  Key3 : Integer;
begin
  Key1:=DataSet.FieldByName('DISC_ID').AsString;
  Key2:=DataSet.FieldByName('TRACK1').AsInteger;
  Key3:=DataSet.FieldByName('TRACK2').AsInteger;
  If tableMusic.FindKey([Key1, Key2, Key3]) then begin
    tableProgWORK_TITLE.AsString :=tableMusic.FieldByName('TITLE').AsString;
    tableProgWORK_PLAYER.AsString:=tableMusic.FieldByName('PLAYER').AsString;
    tableProgWORK_FRAMES.AsInteger:=tableMusic.FieldByName('FRAMES').AsInteger;
  end;
end;


//******************************************************************************
//* Invalidates the dbgridProg object
//******************************************************************************
procedure TformProgPlayer.dbgridProgColExit(Sender: TObject);
begin
  dbgridProg.Invalidate;
end;


//******************************************************************************
//* Sets the directory of the playing program database
//******************************************************************************
procedure TformProgPlayer.FormCreate(Sender: TObject);
const
  CRLF = chr($0D)+chr($0D);
var
  DatabaseName   : string;
  MessageStr     : string;
begin
  try
    mediaplayerCD.Open;
  except
    MessageStr:='Could not open the CD-Audio player.  '+CRLF+
                'It may be occupied by another process.';
    MessageDlg(MessageStr, mtWarning, [mbOK], 0);
    mediaplayerCD.Close;
    formProgPlayer.Release;
  end;
  timer1.Enabled:=True;

  DatabaseName:=ExtractFilePath(ParamStr(0))+'WORK';
  tableProg.Active:=False;
  Main.DisplayStatus('Database name = '+DatabaseName);
  tableProg.Databasename:= DatabaseName;
  Main.DisplayStatus('Activating "tableProg"...');
  tableProg.Active:=True;
  Main.DisplayStatus('Refreshing "tableProg"...');
  tableProg.Refresh;
  Main.DisplayStatus('Showing "Program Player"...');
  Show;
  WindowState:=wsNormal;
end;


//******************************************************************************
//* Colors selected record of the dbgrid
//* Set "DefaultDrawing" propaty as "False".
//******************************************************************************
procedure TformProgPlayer.dbgridProgDrawDataCell(Sender: TObject;
  const Rect: TRect; Field: TField; State: TGridDrawState);
begin
  if gdFocused in State then begin
    dbgridProg.Canvas.Brush.Color:=clAqua;
    dbgridProg.Canvas.Font.Color:=clBlack;
    dbgridProg.Canvas.Font.Style:=[fsBold];
  end;
  dbgridProg.DefaultDrawDataCell(Rect, Field, State);
end;


procedure TformProgPlayer.bitbtnDetailsOfMusicClick(Sender: TObject);
var
     DiscID : string;
     //MusicTitle : string;
     Track1 : Integer;
     Track2 : Integer;
begin
  DiscID:=tableProg.FieldByName('DISC_ID').AsString;
  Track1:=tableProg.FieldByName('TRACK1').AsInteger;
  Track2:=tableProg.FieldByName('TRACK2').AsInteger;
  //MusicTitle:=tableProg.FieldByName('WORK_TITLE').AsString;
  Main.CreateOrShowDetailsOfMusicForm(DiscID, Track1, Track2);
end;

procedure TformProgPlayer.radiogroupPageClick(Sender: TObject);
var
  FilterStr : string;
begin
  case radiogroupPage.Itemindex of
    0: FilterStr:='A';
    1: FilterStr:='B';
    2: FilterStr:='C';
  end;
  FilterStr:='PAGE='+''''+FilterStr+'''';
  Main.DisplayStatus('Filtering: '+FilterStr);
  tableProg.Filter:=FilterStr;
  tableProg.Filtered:=True;
  tableProg.Refresh;
  DisplayMusicTime;
end;

procedure TformProgPlayer.tableProgAfterPost(DataSet: TDataSet);
begin
  DisplayMusicTime;
end;

end.
