///////////////////////////////////////////////////////////////////////////
//                       Alias manager for Hyper Juke                    //
//                Copyright (C) 1990,1997 Fumio Kawamata                 //
//                                                                       //
//  This program is free software; you can redistribute it and/or modify //
//  it under the terms of the GNU General Public License as published by //
//  the Free Software Foundation; version 2 of the License.              //                                                    //
//                                                                       //
//  This program is distributed in the hope that it will be useful, but  //
//  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANT- //
//  ABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General    //
//  Public License in the directory "GNUGPL" for more details.           //
///////////////////////////////////////////////////////////////////////////
//*************************************************************************
//*
//*************************************************************************
unit setaliases;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, FileCtrl, Buttons, DB, DBTables;

type
  TformMain = class(TForm)
    bitbtnOK: TBitBtn;
    bitbtnExit: TBitBtn;
    labelExplanation: TLabel;
    labelCommonDatabase: TLabel;
    labelPersonalDatabase: TLabel;
    editCommonDatabase: TEdit;
    editPersonalDatabase: TEdit;
    tableDummy: TTable;
    bitbtnDelete: TBitBtn;
    procedure bitbtnExitClick(Sender: TObject);
    procedure bitbtnOKClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bitbtnDeleteClick(Sender: TObject);
  private
    procedure SetAlias(AliasName, AliasPath: String);
    procedure DeleteAlias(AliasName: String);
  public
  end;

var
  formMain: TformMain;

implementation

{$R *.DFM}

//*************************************************************************
//* Deletes alias
//*************************************************************************
procedure TformMain.DeleteAlias(AliasName: string);
var
  AliasNameList : TStringList;
  Index         : Integer;
  Matched       : boolean;
begin
  AliasNameList:=TStringList.Create;
  Session.GetAliasNames(AliasNameList);
  Matched:=AliasNameList.Find(AliasName, Index);
  AliasNameList.Free;
  if Matched
    then
      begin
        Session.DeleteAlias(AliasName);
        Session.SaveConfigFile;
        MessageDlg('Deleted '+ AliasName, mtInformation, [mbOK], 0);
      end
    else
      begin
        MessageDlg(AliasName+' is not found.', mtWarning, [mbOK], 0);
      end;
end;


//*************************************************************************
//* Creates or modifys aliases.
//* Creates when it is not existed and modifys when it is existed.
//*************************************************************************
procedure TformMain.SetAlias(AliasName, AliasPath: String);
var
  AliasNameList : TStringList;
  AliasParams   : TStringList;
  Index         : Integer;
  Matched       : boolean;
begin
  AliasNameList:=TStringList.Create;
  Session.GetAliasNames(AliasNameList);
  Matched:=AliasNameList.Find(AliasName, Index);
  AliasNameList.Free;

  if Matched=False
    then
      begin
        Session.AddStandardAlias(AliasName, AliasPath, 'PARADOX');
        Session.SaveConfigFile;
        MessageDlg('Created '+AliasName, mtInformation ,[mbOK],0);
      end
    else
      begin
        AliasParams:=TStringList.Create;
        Session.GetAliasParams(AliasName, AliasParams);
        AliasParams.Values['PATH']:=AliasPath;
        Session.ModifyAlias(AliasName, AliasParams);
        Session.SaveConfigFile;
        AliasParams.Free;
        MessageDlg('Modified '+AliasName, mtInformation ,[mbOK],0);
      end;
end;


//*************************************************************************
//* Executes when Exit button is clicked
//*************************************************************************
procedure TformMain.bitbtnExitClick(Sender: TObject);
begin
  Close;
end;


//*************************************************************************
//* Executes when OK button is clicked
//*************************************************************************
procedure TformMain.bitbtnOKClick(Sender: TObject);
begin
  SetAlias('HYPERJUKE_C', editCommonDatabase.text);
  SetAlias('HYPERJUKE_P', editPersonalDatabase.text);
  MessageDlg('Finished.', mtInformation ,[mbOK],0);
end;


//*************************************************************************
//* Executes when form is created
//*************************************************************************
procedure TformMain.FormCreate(Sender: TObject);
var
  CurrentDrvDir : string;
begin
  CurrentDrvDir:=ExtractFileDir(paramstr(0));
  editCommonDatabase.text:=CurrentDrvDir+'\common';
  editPersonalDatabase.text:=CurrentDrvDir+'\personal';
end;


//*************************************************************************
//* Executes when Delete button is clicked
//*************************************************************************
procedure TformMain.bitbtnDeleteClick(Sender: TObject);
begin
  DeleteAlias('HYPERJUKE_C');
  DeleteAlias('HYPERJUKE_P');
  MessageDlg('Finished.', mtInformation ,[mbOK],0);
end;


end.
